//
//  MyView.m
//  Animations
//
//  Created by Florian on 06.02.13.
//  Copyright (c) 2013 whateva0x29a. All rights reserved.
//

#define PI          3.14159265359f
#define PI_HALF     1.57079632679f
#define PI_THIRD    1.04719755119f
#define PI_SIXTH    0.52359877559f
#define PI_DOUBLE   6.28318530717f

#define R2D(rad)                 ((rad) / PI * 180.0f)
#define D2R(deg)                 ((deg) / 180.0f * PI)

#import "DrawingView.h"
 
static void getAngle(float difX, float difY, float *angle)
{
    // returns as radians (360 degrees equals 2*PI)
    // for our other function it would be easier to return in degrees, but as atanf returns the angle in radians we just leave it like that
    
    // avoding ziro division
    // (little bit of a hack, and not a good programming style, but whateva...)
    if(difX == 0)
        difX += 0.0001f;
    
    if(difY == 0)
        difY += 0.0001f;
    
    
    if ( difX >= 0 && difY >= 0 )
    {
        *angle = atanf( difX / difY );
    }
    else if (difX >= 0 && difY <= 0) 
    {    
        *angle = PI_HALF + atanf(-difY / difX);
    }
    else if(difX <= 0 && difY <= 0)
    {
        *angle = PI + atanf(difX / difY);
    }
    else if (difX <= 0 && difY >= 0)
    {
        *angle = PI_HALF + PI + atanf(difY / -difX);
    }
}

static void getColorForAngle(float angle, int *red, int *green, int *blue)
{
    // returns the color for a given angle (angle can be from 0 to DOUBLE_PI, (360.0 degrees in radians))
    
    
    // make sure angle is between 0 and DOUBLE_PI
    while (angle < 0) 
        angle = angle + PI_DOUBLE;
    
    
    // what's faster?
    
    // nr 1:
    /*while (angle > PI_DOUBLE) 
        angle -= PI_DOUBLE;*/
    
    // nr 2:
    /*angle = (int)(angle*500) % ((int)(PI_DOUBLE*500));
    angle /= 500;*/
    
    // nr 3:
    /*if (angle > PI_DOUBLE)
        angle -= (int)(angle / PI_DOUBLE) * PI_DOUBLE;*/
    
    
    // I'd go for number 3 (but I didn't test it, you're free to do so)
    if (angle > PI_DOUBLE)
        angle -= (int)(angle / PI_DOUBLE) * PI_DOUBLE;
 
    *red = 0, *green = 0, *blue = 0;

    
    // this calculates one of the three color parts (one if always ziro, the other one is always 255)
    // (255.0/60.0) is about 4
    int z = 4 * ((int)(angle / PI * 180.0f)%60);
    
    
    if (angle < PI_THIRD)
    {
        *red = 255;
        *green = z;
    }
    else if (angle < PI_THIRD * 2.0) 
    {
        *red = 255 - z;
        *green = 255;
    }
    else if (angle < PI_THIRD * 3.0)
    {
        *green = 255;
        *blue = z;
    }
    else if (angle < PI_THIRD * 4.0) 
    {
        *green = 255 - z;
        *blue = 255;
    }
    else if (angle < PI_THIRD * 5.0)
    {
        *red = z;
        *blue = 255;
    }
    else
    {       
        *red = 255;
        *blue = 255 - z;
    }
}

@implementation DrawingView


- (id)initWithFrame:(NSRect)_frameRect {	
    
    if ( (self = [super initWithFrame:_frameRect]) ) 
    {
        frameRect	= _frameRect;
    }
    return self;
}

- (void)awakeFromNib 
{
    int width = frameRect.size.width;
    int height = frameRect.size.height;
    int bytesPerRow = width * 3;
    
    unsigned char *data = malloc(bytesPerRow *height);
    
	imageRep = [[NSBitmapImageRep alloc]
							initWithBitmapDataPlanes:&data
							pixelsWide:width 
							pixelsHigh:height
							bitsPerSample:8 
							samplesPerPixel:3 
							hasAlpha:NO 
							isPlanar:NO 
							colorSpaceName:@"NSCalibratedRGBColorSpace" 
							bytesPerRow:bytesPerRow 
							bitsPerPixel:0];
	
    angle = 0.0f;
    direction = -1;
    
    [self paint];
    
	timer	= [NSTimer scheduledTimerWithTimeInterval:0.05 // 20 fps, my computer won't draw faster (maybe because of the bad drawing routine :D)
												  target:self 
												selector:@selector(paint) 
												userInfo:NULL 
												 repeats:YES];//*/

    // timer for changing direction
    [NSTimer scheduledTimerWithTimeInterval:(arc4random()%20)/10.0
                                     target:self 
                                   selector:@selector(changeDirection) 
                                   userInfo:NULL 
                                    repeats:NO]; //*/
    
} 

-(void)dealloc
{
    // free the image data
    unsigned char *data;
    [imageRep getBitmapDataPlanes:&data];
    free(data);
    
    [imageRep release];
    
    [super dealloc];
}
-(void)changeDirection
{
    if (direction < 0)
        direction = 1;
    else 
        direction = -1;
    
    [NSTimer scheduledTimerWithTimeInterval:(arc4random()%20)/10.0
                                     target:self 
                                   selector:@selector(changeDirection) 
                                   userInfo:NULL 
                                    repeats:NO];
}
- (void) paint 
{
    // thats our real paint method, where we draw the picture
    
    // measure frame rate
    clock_t start, ende;
    start = clock();
    
    // moving our "rainbow" in one direction
    if (direction > 0)
        angle += PI_SIXTH / 2; 
    else
        angle -= PI_SIXTH / 2; 
    
    
    int width = frameRect.size.width;
    int height = frameRect.size.height;
    float difX, difY, difYSquare, dif;
    float currentAngle;
    
    int bytesPerRow = width * 3;
    
    unsigned char *data = [imageRep bitmapData];
    
    // color parts
    int red, green, blue;

    // iterate over every single pixel
    
    // go over every row
    for (NSInteger y = 0; y < height; y++) 
    {
        // y distance to centre
        difY = y - height / 2.0;
        
        // we calculate this one time for every following x in this row (= at this y), doesn't make a difference on my system (the compiler propably does the same), but whateva...
        difYSquare = difY * difY;
        
        // in every row go over every pixel
		for (NSInteger x = 0; x < width; x++) 
        { 
            // x distance to centre
            difX = x - width / 2.0;
            
            // overall distance to centre
            dif = sqrtf(difX * difX + difYSquare); // there is a name for that function... I just cannot remember (a^2 + b^2 = c^2)
            // we get the angle current point to centre (in radians)
            getAngle(difX, difY, &currentAngle);
            
            // we get the color for the current angle + rotation + distance to centre
            getColorForAngle((currentAngle+angle+(dif/400.0*PI)), &red, &green, &blue);
            
            // we assign the three color parts
            // you could check before if the color we will change will even be changed (this could be faster, because reading is usally faster than writting, but in this case that doesn't make sense, because we always have a new color everywhere)
            data[y*bytesPerRow+x*3] = (unsigned char)red;
            data[y*bytesPerRow+x*3+1] = (unsigned char)green;
            data[y*bytesPerRow+x*3+2] = (unsigned char)blue;
		}
	}
	
	[self setNeedsDisplay:YES];
    
    ende = clock();
    
    printf("\n%f seconds needed for frame equals %.2f fps\n", (float)(ende-start)/CLOCKS_PER_SEC, \
           1.0/((float)(ende-start)/CLOCKS_PER_SEC));
} 

- (void)drawRect:(NSRect)_rect {
    
	[NSGraphicsContext saveGraphicsState];
	
	[imageRep drawInRect:_rect];
		
	[NSGraphicsContext restoreGraphicsState];
	
}

@end
